@@ -20,14 +20,11 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "ev-application.h"
 #include "ev-utils.h"
 #include "ev-file-helpers.h"
 #include "ev-document-factory.h"
+#include "totem-scrsaver.h"
 
 #include <glib.h>
 #include <glib/gi18n.h>
@@ -36,14 +33,47 @@
 #include <gtk/gtkstock.h>
 #include <gtk/gtkwidget.h>
 #include <gtk/gtkmain.h>
+
+#if WITH_GNOME
 #include <libgnomeui/gnome-client.h>
-#include <string.h>
+#endif
 
 #ifdef ENABLE_DBUS
 #include "ev-application-service.h"
 #include <dbus/dbus-glib-bindings.h>
 #endif
 
+#include <string.h>
+
+static void ev_application_add_icon_path_for_screen (GdkScreen *screen);
+
+struct _EvApplication {
+	GObject base_instance;
+
+	gchar *toolbars_file;
+
+	EggToolbarsModel *toolbars_model;
+
+#ifndef HAVE_GTK_RECENT
+	EggRecentModel  *recent_model;
+#endif
+
+	TotemScrsaver *scr_saver;
+
+	gchar *last_chooser_uri;
+
+#ifdef WITH_GTK_PRINT
+	GtkPrintSettings *print_settings;
+#if GTK_CHECK_VERSION (2, 11, 0)
+	gchar            *print_settings_file;
+#endif
+#endif
+};
+
+struct _EvApplicationClass {
+	GObjectClass base_class;
+};
+
 G_DEFINE_TYPE (EvApplication, ev_application, G_TYPE_OBJECT);
 
 #define APPLICATION_SERVICE_NAME "org.gnome.evince.ApplicationService"
@@ -101,6 +131,13 @@ ev_application_register_service (EvApplication *application)
 }
 #endif /* ENABLE_DBUS */
 
+/**
+ * ev_application_get_instance:
+ *
+ * Checks for #EvApplication instance, if it doesn't exist it does create it.
+ *
+ * Returns: an instance of the #EvApplication data.
+ */
 EvApplication *
 ev_application_get_instance (void)
 {
@@ -113,6 +150,7 @@ ev_application_get_instance (void)
 	return instance;
 }
 
+#if WITH_GNOME
 static void
 removed_from_session (GnomeClient *client, EvApplication *application)
 {
@@ -160,7 +198,17 @@ init_session (EvApplication *application)
 	g_signal_connect (client, "die",
 			  G_CALLBACK (removed_from_session), application);
 }
+#endif
 
+/**
+ * ev_display_open_if_needed:
+ * @name: the name of the display to be open if it's needed.
+ *
+ * Search among all the open displays if any of them have the same name as the
+ * passed name. If the display isn't found it tries the open it.
+ *
+ * Returns: a #GdkDisplay of the display with the passed name.
+ */
 static GdkDisplay *
 ev_display_open_if_needed (const gchar *name)
 {
@@ -184,6 +232,17 @@ ev_display_open_if_needed (const gchar *name)
 	return display != NULL ? display : gdk_display_open (name);
 }
 
+/**
+ * get_screen_from_args:
+ * @args: a #GHashTable with data passed to the application.
+ *
+ * Looks for the screen in the display available in the hash table passed to the
+ * application. If the display isn't opened, it's opened and the #GdkScreen
+ * assigned to the screen in that display returned.
+ *
+ * Returns: the #GdkScreen assigned to the screen on the display indicated by
+ *          the data on the #GHashTable.
+ */
 static GdkScreen *
 get_screen_from_args (GHashTable *args)
 {
@@ -212,6 +271,16 @@ get_screen_from_args (GHashTable *args)
 	return screen;
 }
 
+/**
+ * get_window_run_mode_from_args:
+ * @args: a #GHashTable with data passed to the application.
+ *
+ * It does look if the mode option has been passed from command line, using it
+ * as the window run mode, otherwise the run mode will be the normal mode.
+ *
+ * Returns: The window run mode passed from command line or
+ *          EV_WINDOW_MODE_NORMAL in other case.
+ */
 static EvWindowRunMode
 get_window_run_mode_from_args (GHashTable *args)
 {
@@ -228,6 +297,16 @@ get_window_run_mode_from_args (GHashTable *args)
 	return mode;
 }
 
+/**
+ * get_destination_from_args:
+ * @args: a #GHashTable with data passed to the application.
+ *
+ * It does look for the page-label argument parsed from the command line and
+ * if it does exist, it returns an #EvLinkDest.
+ *
+ * Returns: An #EvLinkDest to page-label if it has been passed from the command
+ *          line, NULL in other case.
+ */
 static EvLinkDest *
 get_destination_from_args (GHashTable *args)
 {
@@ -247,6 +326,16 @@ get_destination_from_args (GHashTable *args)
 	return dest;
 }
 
+/**
+ * get_unlink_temp_file_from_args:
+ * @args: a #GHashTable with data passed to the application.
+ *
+ * It does look if the unlink-temp-file option has been passed from the command
+ * line returning it's boolean representation, otherwise it does return %FALSE.
+ *
+ * Returns: the boolean representation of the unlink-temp-file value or %FALSE
+ *          in other case.
+ */
 static gboolean
 get_unlink_temp_file_from_args (GHashTable *args)
 {
@@ -263,6 +352,35 @@ get_unlink_temp_file_from_args (GHashTable *args)
 	return unlink_temp_file;
 }
 
+static const gchar *
+get_print_settings_from_args (GHashTable *args)
+{
+	const gchar *print_settings = NULL;
+	GValue      *value = NULL;
+
+	g_assert (args != NULL);
+
+	value = g_hash_table_lookup (args, "print-settings");
+	if (value) {
+		print_settings = g_value_get_string (value);
+	}
+
+	return print_settings;
+}
+
+/**
+ * ev_application_open_window:
+ * @application: The instance of the application.
+ * @args: A #GHashTable with the arguments data.
+ * @timestamp: Current time value.
+ * @error: The #GError facility.
+ * 
+ * Creates a new window and if the args are available, it's not NULL, it gets
+ * the screen from them and assigns the just created window to it. At last it
+ * does show it.
+ *
+ * Returns: %TRUE.
+ */
 gboolean
 ev_application_open_window (EvApplication  *application,
 			    GHashTable     *args,
@@ -279,6 +397,7 @@ ev_application_open_window (EvApplication  *application,
 	if (screen) {
 		gtk_window_set_screen (GTK_WINDOW (new_window), screen);
 	}
+	ev_application_add_icon_path_for_screen (screen);
 	
 	gtk_widget_show (new_window);
 	
@@ -287,6 +406,16 @@ ev_application_open_window (EvApplication  *application,
 	return TRUE;
 }
 
+/**
+ * ev_application_get_empty_window:
+ * @application: The instance of the application.
+ * @screen: The screen where the empty window will be search.
+ *
+ * It does look if there is any empty window in the indicated screen.
+ *
+ * Returns: The first empty #EvWindow in the passed #GdkScreen or NULL in other
+ *          case.
+ */
 static EvWindow *
 ev_application_get_empty_window (EvApplication *application,
 				 GdkScreen     *screen)
@@ -310,6 +439,18 @@ ev_application_get_empty_window (EvApplication *application,
 	return empty_window;
 }
 
+/**
+ * ev_application_get_uri_window:
+ * @application: The instance of the application.
+ * @uri: The uri to be opened.
+ *
+ * It looks in the list of the windows for the one with the document represented
+ * by the passed uri on it. If the window is empty or the document isn't present
+ * on any window, it will return NULL.
+ *
+ * Returns: The #EvWindow where the document represented by the passed uri is
+ *          shown, NULL in other case.
+ */
 static EvWindow *
 ev_application_get_uri_window (EvApplication *application, const char *uri)
 {
@@ -336,20 +477,11 @@ ev_application_get_uri_window (EvApplication *application, const char *uri)
 	return uri_window;
 }
 
-void
-ev_application_open_uri_at_dest (EvApplication  *application,
-				 const char     *uri,
-				 GdkScreen      *screen,
-				 EvLinkDest     *dest,
-				 EvWindowRunMode mode,
-				 gboolean        unlink_temp_file,
-				 guint           timestamp)
+static void
+ev_application_add_icon_path_for_screen (GdkScreen *screen)
 {
-	EvWindow     *new_window;
 	GtkIconTheme *icon_theme;
 
-	g_return_if_fail (uri != NULL);
-
 	icon_theme = gtk_icon_theme_get_for_screen (screen);
 	if (icon_theme) {
 		gchar **path = NULL;
@@ -374,6 +506,33 @@ ev_application_open_uri_at_dest (EvApplication  *application,
 		g_free (ev_icons_path);
 		g_strfreev (path);
 	}	
+}
+
+/**
+ * ev_application_open_uri_at_dest:
+ * @application: The instance of the application.
+ * @uri: The uri to be opened.
+ * @screen: Thee screen where the link will be shown.
+ * @dest: The #EvLinkDest of the document.
+ * @mode: The run mode of the window.
+ * @unlink_temp_file: The unlink_temp_file option value.
+ * @timestamp: Current time value.
+ */
+void
+ev_application_open_uri_at_dest (EvApplication  *application,
+				 const char     *uri,
+				 GdkScreen      *screen,
+				 EvLinkDest     *dest,
+				 EvWindowRunMode mode,
+				 gboolean        unlink_temp_file,
+				 const gchar    *print_settings, 
+				 guint           timestamp)
+{
+	EvWindow *new_window;
+
+	g_return_if_fail (uri != NULL);
+	
+	ev_application_add_icon_path_for_screen (screen);
 
 	new_window = ev_application_get_uri_window (application, uri);
 	
@@ -390,7 +549,8 @@ ev_application_open_uri_at_dest (EvApplication  *application,
 
 	/* We need to load uri before showing the window, so
 	   we can restore window size without flickering */	
-	ev_window_open_uri (new_window, uri, dest, mode, unlink_temp_file);
+	ev_window_open_uri (new_window, uri, dest, mode,
+			    unlink_temp_file, print_settings);
 
 	ev_document_fc_mutex_lock ();
 	gtk_widget_show (GTK_WIDGET (new_window));
@@ -400,6 +560,14 @@ ev_application_open_uri_at_dest (EvApplication  *application,
 				      timestamp);
 }
 
+/**
+ * ev_application_open_uri:
+ * @application: The instance of the application.
+ * @uri: The uri to be opened
+ * @args: A #GHashTable with the arguments data.
+ * @timestamp: Current time value.
+ * @error: The #GError facility.
+ */
 gboolean
 ev_application_open_uri (EvApplication  *application,
 			 const char     *uri,
@@ -410,6 +578,7 @@ ev_application_open_uri (EvApplication  *application,
 	EvLinkDest      *dest = NULL;
 	EvWindowRunMode  mode = EV_WINDOW_MODE_NORMAL;
 	gboolean         unlink_temp_file = FALSE;
+	const gchar     *print_settings = NULL;
 	GdkScreen       *screen = NULL;
 
 	if (args) {
@@ -418,11 +587,12 @@ ev_application_open_uri (EvApplication  *application,
 		mode = get_window_run_mode_from_args (args);
 		unlink_temp_file = (mode == EV_WINDOW_MODE_PREVIEW &&
 				    get_unlink_temp_file_from_args (args));
+		print_settings = get_print_settings_from_args (args);
 	}
 	
 	ev_application_open_uri_at_dest (application, uri, screen,
-					 dest, mode, unlink_temp_file, 
-					 timestamp);
+					 dest, mode, unlink_temp_file,
+					 print_settings, timestamp);
 
 	if (dest)
 		g_object_unref (dest);
@@ -440,8 +610,8 @@ ev_application_open_uri_list (EvApplication *application,
 
 	for (l = uri_list; l != NULL; l = l->next) {
 		ev_application_open_uri_at_dest (application, (char *)l->data,
-						 screen, NULL, 0, FALSE, 
-						 timestamp);
+						 screen, NULL, 0, FALSE,
+						 NULL, timestamp);
 	}
 }
 
@@ -450,10 +620,8 @@ ev_application_shutdown (EvApplication *application)
 {
 	if (application->toolbars_model) {
 		g_object_unref (application->toolbars_model);
-		g_object_unref (application->preview_toolbars_model);
 		g_free (application->toolbars_file);
 		application->toolbars_model = NULL;
-		application->preview_toolbars_model = NULL;
 		application->toolbars_file = NULL;
 	}
 
@@ -463,6 +631,35 @@ ev_application_shutdown (EvApplication *application)
 		application->recent_model = NULL;
 	}
 #endif
+
+#ifdef WITH_GTK_PRINT
+#if GTK_CHECK_VERSION (2, 11, 0)
+	if (application->print_settings_file) {
+		if (application->print_settings) {
+			GError *error = NULL;
+			
+			gtk_print_settings_to_file (application->print_settings,
+						    application->print_settings_file,
+						    &error);
+			if (error) {
+				g_warning (error->message);
+				g_error_free (error);
+			}
+
+			g_object_unref (application->print_settings);
+			application->print_settings = NULL;
+		}
+
+		g_free (application->print_settings_file);
+		application->print_settings_file = NULL;
+	}
+#else /* ! GTK 2.11.0 */
+	if (application->print_settings) {
+		g_object_unref (application->print_settings);
+		application->print_settings = NULL;
+	}
+#endif /* GTK 2.11.0 */
+#endif /* WITH_GTK_PRINT */
 	
 	g_free (application->last_chooser_uri);
 	g_object_unref (application);
@@ -478,7 +675,9 @@ ev_application_class_init (EvApplicationClass *ev_application_class)
 static void
 ev_application_init (EvApplication *ev_application)
 {
+#if WITH_GNOME
 	init_session (ev_application);
+#endif
 
 	ev_application->toolbars_model = egg_toolbars_model_new ();
 
@@ -497,14 +696,6 @@ ev_application_init (EvApplication *ev_application)
 	egg_toolbars_model_set_flags (ev_application->toolbars_model, 0,
 				      EGG_TB_MODEL_NOT_REMOVABLE); 
 
-	ev_application->preview_toolbars_model = egg_toolbars_model_new ();
-
-	egg_toolbars_model_load_toolbars (ev_application->preview_toolbars_model,
-					  DATADIR"/evince-preview-toolbar.xml");
-
-	egg_toolbars_model_set_flags (ev_application->preview_toolbars_model, 0,
-				      EGG_TB_MODEL_NOT_REMOVABLE); 
-
 #ifndef HAVE_GTK_RECENT
 	ev_application->recent_model = egg_recent_model_new (EGG_RECENT_MODEL_SORT_MRU);
 	/* FIXME we should add a mime type filter but current eggrecent
@@ -516,6 +707,14 @@ ev_application_init (EvApplication *ev_application)
 #endif /* HAVE_GTK_RECENT */
 }
 
+/**
+ * ev_application_get_windows:
+ * @application: The instance of the application.
+ *
+ * It creates a list of the top level windows.
+ *
+ * Returns: A #GList of the top level windows.
+ */
 GList *
 ev_application_get_windows (EvApplication *application)
 {
@@ -535,45 +734,99 @@ ev_application_get_windows (EvApplication *application)
 	return windows;
 }
 
-EggToolbarsModel *ev_application_get_toolbars_model (EvApplication *application,
-						     gboolean preview)
+EggToolbarsModel *
+ev_application_get_toolbars_model (EvApplication *application)
 {
-	return preview ? 
-	    application->preview_toolbars_model : application->toolbars_model;
+	return application->toolbars_model;
 }
 
 #ifndef HAVE_GTK_RECENT
-EggRecentModel *ev_application_get_recent_model (EvApplication *application)
+EggRecentModel *
+ev_application_get_recent_model (EvApplication *application)
 {
 	return application->recent_model;
 }
 #endif
 
-void ev_application_save_toolbars_model (EvApplication *application)
+void
+ev_application_save_toolbars_model (EvApplication *application)
 {
         egg_toolbars_model_save_toolbars (application->toolbars_model,
 			 	          application->toolbars_file, "1.0");
 }
 
-void ev_application_set_chooser_uri (EvApplication *application, const gchar *uri)
+void
+ev_application_set_chooser_uri (EvApplication *application, const gchar *uri)
 {
 	g_free (application->last_chooser_uri);
 	application->last_chooser_uri = g_strdup (uri);
 }
 
-const gchar* ev_application_get_chooser_uri (EvApplication *application)
+const gchar *
+ev_application_get_chooser_uri (EvApplication *application)
 {
 	return application->last_chooser_uri;
 }
 
-void ev_application_screensaver_enable  (EvApplication   *application)
+void
+ev_application_screensaver_enable (EvApplication *application)
 {
 	if (application->scr_saver)
 		totem_scrsaver_enable (application->scr_saver);	
 }
 
-void ev_application_screensaver_disable (EvApplication   *application)
+void
+ev_application_screensaver_disable (EvApplication *application)
 {
 	if (application->scr_saver)
 		totem_scrsaver_disable (application->scr_saver);	
 }
+
+#ifdef WITH_GTK_PRINT
+GtkPrintSettings *
+ev_application_get_print_settings (EvApplication *application)
+{
+	if (application->print_settings)
+		return application->print_settings;
+	
+#if GTK_CHECK_VERSION (2, 11, 0)
+	if (!application->print_settings_file) {
+		application->print_settings_file =
+			g_build_filename (ev_dot_dir (), "print-settings", NULL);
+	}
+
+	if (g_file_test (application->print_settings_file, G_FILE_TEST_IS_REGULAR)) {
+		GError *error = NULL;
+		
+		application->print_settings =
+			gtk_print_settings_new_from_file (application->print_settings_file, &error);
+		
+		if (error) {
+			g_warning (error->message);
+			g_error_free (error);
+		} else {
+			return application->print_settings;
+		}
+	}
+#endif /* GTK 2.11.0 */
+	
+	application->print_settings = gtk_print_settings_new ();
+
+	return application->print_settings;
+}
+
+void
+ev_application_set_print_settings (EvApplication    *application,
+				   GtkPrintSettings *settings)
+{
+	g_return_if_fail (GTK_IS_PRINT_SETTINGS (settings));
+	
+	if (settings == application->print_settings)
+		return;
+
+	if (application->print_settings)
+		g_object_unref (application->print_settings);
+	
+	application->print_settings = g_object_ref (settings);
+}
+#endif /* WITH_GTK_PRINT */
