@@ -59,9 +59,6 @@ struct _EvMetadataManager
 	gboolean	 values_loaded; /* It is true if the file 
 					   has been read */
 
-	gboolean	 modified;	/* It is true if the file 
-					   has top be written */
-
 	guint 		 timeout_id;
 
 	GHashTable	*items;
@@ -72,6 +69,12 @@ static gboolean ev_metadata_manager_save (gpointer data);
 
 static EvMetadataManager *ev_metadata_manager = NULL;
 
+/**
+ * item_free:
+ * @data: a pointer to a #Item data
+ *
+ * It does free the values on the #GHashTable where data points.
+ */
 static void
 item_free (gpointer data)
 {
@@ -87,20 +90,16 @@ item_free (gpointer data)
 	g_free (item);
 }
 
-void
-ev_metadata_manager_init (void)
+/**
+ * ev_metadata_arm_timeout
+ *
+ * Setup a timeout for saving the metadata to disk.
+ */
+static void
+ev_metadata_arm_timeout(void)
 {
-	ev_metadata_manager = g_new0 (EvMetadataManager, 1);
-
-	ev_metadata_manager->values_loaded = FALSE;
-	ev_metadata_manager->modified = FALSE;
-
-	ev_metadata_manager->items = 
-		g_hash_table_new_full (g_str_hash, 
-				       g_str_equal, 
-				       g_free,
-				       item_free);
-
+	if (ev_metadata_manager->timeout_id)
+		return;
 	ev_metadata_manager->timeout_id = 
 		g_timeout_add_full (G_PRIORITY_DEFAULT_IDLE,
 				    2000, /* 2 sec */
@@ -109,6 +108,29 @@ ev_metadata_manager_init (void)
 				    NULL);
 }
 
+/**
+ * ev_metadata_manager_init:
+ *
+ * Creates an EvMetadataManager with default values.
+ *
+ *  values_loaded   ->  %FALSE.
+ *  timeout_id      ->  the id of the event source.
+ *  items           ->  a new full empty #GHashTable.
+ */
+void
+ev_metadata_manager_init (void)
+{
+	ev_metadata_manager = g_new0 (EvMetadataManager, 1);
+
+	ev_metadata_manager->values_loaded = FALSE;
+
+	ev_metadata_manager->items = 
+		g_hash_table_new_full (g_str_hash, 
+				       g_str_equal, 
+				       g_free,
+				       item_free);
+}
+
 /* This function must be called before exiting ev */
 void
 ev_metadata_manager_shutdown (void)
@@ -116,9 +138,11 @@ ev_metadata_manager_shutdown (void)
 	if (ev_metadata_manager == NULL)
 		return;
 
-	g_source_remove (ev_metadata_manager->timeout_id);
-
-	ev_metadata_manager_save (NULL);
+	if (ev_metadata_manager->timeout_id) {
+		g_source_remove (ev_metadata_manager->timeout_id);
+		ev_metadata_manager->timeout_id = 0;
+		ev_metadata_manager_save (NULL);
+	}
 
 	if (ev_metadata_manager->items != NULL)
 		g_hash_table_destroy (ev_metadata_manager->items);
@@ -375,12 +399,12 @@ ev_metadata_manager_set_last (const gchar *key,
 	}
 
 	item->atime = time (NULL);
-	ev_metadata_manager->modified = TRUE;
+	ev_metadata_arm_timeout ();
 	return;
 }
 				 
 /**
- * ev_metadata_manager_set:
+ * ev_metadata_manager_get:
  * @uri: Uri to set data for, if @NULL, we return default value
  * @key: Key to set uri
  * @value: GValue struct filled up with value
@@ -510,7 +534,7 @@ ev_metadata_manager_set (const gchar  *uri,
 
 	item->atime = time (NULL);
 
-	ev_metadata_manager->modified = TRUE;
+	ev_metadata_arm_timeout ();
 }
 
 static void
@@ -631,8 +655,7 @@ ev_metadata_manager_save (gpointer data)
 	xmlNodePtr root;
 	gchar *file_name;
 
-	if (!ev_metadata_manager->modified)
-		return TRUE;
+	ev_metadata_manager->timeout_id = 0;
 
 	resize_items ();
 		
@@ -656,9 +679,7 @@ ev_metadata_manager_save (gpointer data)
 	
 	xmlFreeDoc (doc); 
 
-	ev_metadata_manager->modified = FALSE;
-
-	return TRUE;
+	return FALSE;
 }
 
 void
